import configparser
from dataclasses import dataclass, is_dataclass, asdict
from datetime import datetime
import json

from pathlib import Path
from typing import List, NamedTuple

from reflectionist import DB_READ_ERROR, DB_WRITE_ERROR, JSON_ERROR, SUCCESS

DEFAULT_DB_FILE_PATH = Path.home().joinpath(
    "." + Path.home().stem + "_reflections.json"
)


@dataclass
class Reflection:
    created_at: str
    happened: str
    feeling: str
    learned: str


class CurrentReflection(NamedTuple):
    reflection: Reflection
    return_code: int


def get_database_path(config_file: Path) -> Path:
    """Return the current path to the reflections database."""
    config_parser = configparser.ConfigParser()
    config_parser.read(config_file)
    return Path(config_parser["General"]["database"])


def init_database(db_path: Path) -> int:
    """Create the reflections database."""
    try:
        db_path.write_text("[]")  # Empty reflections list
        return SUCCESS
    except OSError:
        return DB_WRITE_ERROR


@dataclass
class DBResponse:
    reflections: List[Reflection]
    return_code: int


class DataclassJSONEncoder(json.JSONEncoder):
    def default(self, o):
        if is_dataclass(o):
            return asdict(o)
        return super().default(o)


class DatabaseHandler:
    def __init__(self, db_path: Path) -> None:
        self._db_path = db_path

    def read(self) -> DBResponse:
        try:
            with self._db_path.open("r") as db:
                try:
                    return DBResponse(json.load(db), SUCCESS)
                except json.JSONDecodeError:
                    return DBResponse([], JSON_ERROR)
        except OSError:
            return DBResponse([], DB_READ_ERROR)

    def write(self, reflections: List[Reflection]) -> DBResponse:
        try:
            with self._db_path.open("w") as db:
                json.dump(reflections, db, indent=4, cls=DataclassJSONEncoder)
            return DBResponse(reflections, SUCCESS)
        except OSError:
            return DBResponse(reflections, DB_WRITE_ERROR)



import configparser
from pathlib import Path

import typer

from reflectionist import DB_WRITE_ERROR, DIR_ERROR, FILE_ERROR, SUCCESS, __app_name__

CONFIG_DIR_PATH = Path(typer.get_app_dir(__app_name__))
CONFIG_FILE_PATH = CONFIG_DIR_PATH / "config.ini"


def init_app(db_path: str) -> int:
    """Initialize the application."""
    config_code = _init_config_file()
    if config_code != SUCCESS:
        return config_code
    database_code = _create_database(db_path)
    if database_code != SUCCESS:
        return database_code
    return SUCCESS


def _init_config_file() -> int:
    try:
        CONFIG_DIR_PATH.mkdir(exist_ok=True)
    except OSError:
        return DIR_ERROR
    try:
        CONFIG_FILE_PATH.touch(exist_ok=True)
    except OSError:
        return FILE_ERROR
    return SUCCESS


def _create_database(db_path: str) -> int:
    config_parser = configparser.ConfigParser()
    config_parser["General"] = {"database": db_path}
    try:
        with CONFIG_FILE_PATH.open("w") as file:
            config_parser.write(file)
    except OSError:
        return DB_WRITE_ERROR
    return SUCCESS



from datetime import datetime
from pathlib import Path
from typing import List
from reflectionist import DB_READ_ERROR, SUCCESS
from uuid import uuid4

from reflectionist.database import CurrentReflection, DatabaseHandler, Reflection


class ReadException(Exception):
    pass


class Service:
    def __init__(self, db_path: Path) -> None:
        self._db_handler = DatabaseHandler(db_path)

    def add(self, happened: str, felt: str, learned: str) -> CurrentReflection:
        reflection = Reflection(str(datetime.utcnow()), happened, felt, learned)
        read = self._db_handler.read()
        if read.return_code == DB_READ_ERROR:
            return CurrentReflection(reflection, read.return_code)
        read.reflections.insert(0, reflection)
        write = self._db_handler.write(read.reflections)
        return CurrentReflection(reflection, write.return_code)

    def get(self) -> List[Reflection]:
        read = self._db_handler.read()
        return read.reflections

    def describe(self, id: int) -> Reflection:
        response = self._db_handler.read()
        if response.return_code != SUCCESS:
            raise ReadException(
                f"Could not read reflections, error: {response.return_code}"
            )
        if id < 0 or id >= len(response.reflections):
            raise IndexError(f"Reflection with id {id} does not exist.")
        return response.reflections[id]



from pathlib import Path
from typing import Optional
import typer

from reflectionist import ERRORS, config, database, __version__, __app_name__
from reflectionist.service import Service

app = typer.Typer()


def get_service() -> Service:
    if config.CONFIG_FILE_PATH.exists():
        db_path = database.get_database_path(config.CONFIG_FILE_PATH)
    else:
        typer.secho(
            'Config file not found. Please, run "reflectionist init"',
            fg=typer.colors.RED,
        )
        raise typer.Exit(1)
    if db_path.exists():
        return Service(db_path)
    else:
        typer.secho(
            'Database not found. Please, run "reflectionist init"',
            fg=typer.colors.RED,
        )
        raise typer.Exit(1)


def _version_callback(value: bool) -> None:
    if value:
        typer.echo(f"{__app_name__} v{__version__}")
        raise typer.Exit()


@app.command()
def init(
    db_path: str = typer.Option(
        str(database.DEFAULT_DB_FILE_PATH),
        "--db-path",
        "-db",
        prompt="reflections folder path",
    ),
) -> None:
    """Initialize the reflections database."""
    app_init_error = config.init_app(db_path)
    if app_init_error:
        typer.secho(
            f'Creating config file failed with "{ERRORS[app_init_error]}"',
            fg=typer.colors.RED,
        )
        raise typer.Exit(1)
    db_init_error = database.init_database(Path(db_path))
    if db_init_error:
        typer.secho(
            f'Creating database failed with "{ERRORS[db_init_error]}"',
            fg=typer.colors.RED,
        )
        raise typer.Exit(1)
    else:
        typer.secho(f"The reflections database is {db_path}", fg=typer.colors.GREEN)


@app.command()
def create(
    happened: str = typer.Option(
        default=None, prompt="What happened that affected me?"
    ),
    felt: str = typer.Option(default=None, prompt="How did I feel then and now?"),
    learned: str = typer.Option(default=None, prompt="What did I learn about myself?"),
) -> None:
    """Create a new reflection."""
    service = get_service()
    _, return_code = service.add(happened=happened, felt=felt, learned=learned)
    if return_code != database.SUCCESS:
        typer.secho(
            f'Creating a reflection failed with "{ERRORS[return_code]}"',
            fg=typer.colors.RED,
        )
        raise typer.Exit(1)
    else:
        typer.secho(
            "Reflection added!",
            fg=typer.colors.GREEN,
        )


@app.command()
def list() -> None:
    """List all reflections."""
    service = get_service()
    reflections = service.get()
    if len(reflections) == 0:
        typer.secho("There are no reflections yet", fg=typer.colors.RED)
        raise typer.Exit()
    typer.secho(f"{len(reflections)} reflections:", fg=typer.colors.BLUE)
    for i, reflection in enumerate(reflections):
        typer.secho(f"{i}. situation={reflection['happened']}", fg=typer.colors.BLUE)


@app.command()
def describe(id: int = typer.Option(default=None, prompt="Which reflection?")) -> None:
    """Describe a reflection given an id."""
    service = get_service()
    reflection = service.describe(id=id)
    typer.secho(reflection, fg=typer.colors.BLUE)


@app.callback()
def main(
    version: Optional[bool] = typer.Option(
        None,
        "--version",
        "-v",
        help="Show the app version and exit.",
        callback=_version_callback,
        is_eager=True,
    )
) -> None:
    return


if __name__ == "__main__":
    app()



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['reflectionist']

package_data = \
{'': ['*']}

install_requires = \
['click>=8.0.3,<9.0.0', 'typer>=0.4.0,<0.5.0']

entry_points = \
{'console_scripts': ['reflectionist = reflectionist.cli:app']}

setup_kwargs = {
    'name': 'reflectionist',
    'version': '0.1.2',
    'description': 'A simple CLI app that helps you to track your reflections during your day.',
    'long_description': "\n# reflectionist\n\nA simple CLI app that helps you to track your reflections during your day.\n\n\n# Why?\nBeing capable of reflecting helps to understand stressful situations.\n\nHaving a different perspective on what happened facilitates to take actions on how to behave to avoid the same feeling again reducing your stress levels and enabling personal growth!\n\n## Reflection questions\n\nThe following questions were extracted from my experience at Hyper Island, during the master's in Digital Management.\n\n\n    1. What happened that affected me?\n    2. How did I feel then and now?\n    3. What did I learn about myself?\n\n\n\n\n## Installation\n\nInstall reflectionist with pip\n\n```bash\n  pip install reflectionist\n```\n    \n## Features\n\n- Init environment: `reflectionist init`\n- Add reflection: `reflectionist create`\n- List reflections: `reflectionist list`\n- Describe specific reflection: `reflectionist describe`\n\n\n## Contributing\n\nContributions are always welcome! Just open a new issue that I will check it ASAP.\n\n",
    'author': 'Dênis Araújo da Silva',
    'author_email': 'silvadenisaraujo@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)
